import { type Player, type PlayerPlugin, type RequiredPartial, type Source } from '@oplayer/core';
import type { BitrateInfo, MediaPlayerClass, MediaPlayerSettingClass, ProtectionDataSet } from 'dashjs';
export type Matcher = (video: HTMLVideoElement, source: Source) => boolean;
export interface DashPluginOptions {
    library?: string;
    matcher?: Matcher;
    /**
     * default auto
     */
    defaultQuality?: (levels: BitrateInfo[]) => number;
    /**
     * default browser language
     */
    defaultAudio?: (tracks: dashjs.MediaInfo[]) => number;
    /**
     * default browser language
     */
    defaultSubtitle?: (tracks: dashjs.MediaInfo[]) => number;
    /**
     * config for dashjs
     *
     * @type {MediaPlayerSettingClass}
     */
    config?: MediaPlayerSettingClass;
    drm?: ProtectionDataSet;
    /**
     * enable quality control for the stream, does not apply to the native (iPhone) clients.
     * @default: true
     */
    qualityControl?: boolean;
    /**
     *  control how the stream quality is switched. default: immediate
     *  @value immediate: Trigger an immediate quality level switch to new quality level. This will abort the current fragment request if any, flush the whole buffer, and fetch fragment matching with current position and requested quality level.
     *  @value smooth: Trigger a quality level switch for next fragment. This could eventually flush already buffered next fragment.
     */
    qualitySwitch?: 'immediate' | 'smooth';
    /**
     * @default: false
     */
    withBitrate?: boolean;
    /**
     * @default: true
     */
    audioControl?: boolean;
    /**
     * @default: true
     */
    textControl?: boolean;
}
declare class DashPlugin implements PlayerPlugin {
    key: string;
    name: string;
    version: string;
    static library: typeof import('dashjs');
    player: Player;
    instance?: MediaPlayerClass;
    options: RequiredPartial<DashPluginOptions, 'config' | 'library' | 'drm'>;
    constructor(options?: DashPluginOptions);
    apply(player: Player): this;
    load({ $video }: Player, source: Source): Promise<false | this>;
    destroy(): void;
}
export default function create(options?: DashPluginOptions): DashPlugin;
export {};
//# sourceMappingURL=index.d.ts.map